unit MainFrm;
{------------------------------------------------------------------------------}
(*
  This project shows how to find all occurrences of text inside {} and apply
  a special (protected) style to them.
  v1.1

  History:
  v1.0 - initial release
  v1.1 - text processed in Unicode
*)
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  RVScroll, RichView, RVStyle, RVTable, CRVData, 
  RVEdit, RVUni, RVInsertItems, StdCtrls, RVTypes;

{$I RV_Defs.inc}

type


  TForm1 = class(TForm)
    rvsTemplate: TRVStyle;
    Label2: TLabel;
    rve: TRichViewEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    procedure CreateTemplate;
    procedure ProtectFields(RVData: TCustomRVData);
    procedure ProtectField(RVData: TCustomRVData;
      var ItemNo, Pos: Integer; const FieldName: TRVUnicodeString);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}
{ Creating a sample template }
procedure TForm1.CreateTemplate;
begin
   rve.Clear;
   rve.AddNL('To {NAME}', 0, 0);
   rve.AddNL('{FULL_ADDRESS}', 0, 0);
   rve.AddNL('Letter Sample', 1, 1);
   rve.AddNL('Hello {NAME}', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('You can find that useless letters sometimes make sense.', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('With best regards,', 0, 0);
   rve.AddNL('your template generator.', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('', 0, 0);
   rve.Format;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateTemplate;
end;
{------------------------------------------------------------------------------}
procedure TForm1.Button1Click(Sender: TObject);
begin
  ProtectFields(rve.RVData);
  rve.Format;
end;
{==============================================================================}
(*
  This function iterates through all items in RVData starting from StartItemNo,
  searches for strings inside {}, calls ProtectField()
  You can move this function to your application unchanged.
*)
procedure TForm1.ProtectFields(RVData: TCustomRVData);
var i,j,r,c: Integer;
    table: TRVTableItemInfo;
    FieldName: TRVUnicodeString;
    s: TRVUnicodeString;
    FieldStart: Integer;
begin
  i := 0;
  while i<RVData.ItemCount do
  begin
    if RVData.GetItem(i) is TRVTableItemInfo then
    begin // recursive processing of tables
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.RowCount-1 do
        for c := 0 to table.ColCount-1 do
          if table.Cells[r,c]<>nil then
            ProtectFields(table.Cells[r,c].GetRVData);
    end
    else if RVData.GetItemStyle(i)>=0 then
    begin
      s := RVData.GetItemTextW(i);
      FieldStart := 0;
      j := 1;
      while j<=Length(s) do
      begin
        if s[j]='{' then // may be this is a beginning of field code
          FieldStart := j
        else if (s[j]='}') and (FieldStart>0) then
        begin // field code is found
          FieldName :=  Copy(s, FieldStart+1, j-FieldStart-1);
          // deleting field code
          Delete(s, FieldStart, j-FieldStart+1);
          RVData.SetItemTextW(i, s);
          // inserting field value
          j := FieldStart;
          ProtectField(RVData, i, j, FieldName);
          if RVData.GetItemStyle(i)<0 then
            s := ''
          else
            s := RVData.GetItemTextW(i);
          FieldStart := 0;
          continue;
        end;
        inc(j);
      end;
    end;
    inc(i);
  end;
end;
{------------------------------------------------------------------------------}
function GetProtectedStyle(RVStyle: TRVStyle; StyleNo: Integer): Integer;
var TextStyle: TFontInfo;
begin
  TextStyle := TFontInfo.Create(nil);
  try
    TextStyle.Assign(RVStyle.TextStyles[StyleNo]);
    TextStyle.Protection := [rvprModifyProtect,rvprConcateProtect,rvprDoNotAutoSwitch,rvprStyleProtect];
    TextStyle.BackColor := clSilver ;
    Result := RVStyle.FindTextStyle(TextStyle);
  finally
    TextStyle.Free;
  end;
end;
{------------------------------------------------------------------------------}
procedure TForm1.ProtectField(RVData: TCustomRVData;
  var ItemNo, Pos: Integer; const FieldName: TRVUnicodeString);
var s1, s2: String;
    ParaNo, StyleNo: Integer;
    BR, PageBreak: Boolean;
    Tag: TRVTag;
begin
  s1 := RVData.GetItemTextW(ItemNo);
  s2 := Copy(s1, Pos, MaxInt);
  s1 := Copy(s1, 1, Pos-1);
  ParaNo := -1;
  BR := False;
  PageBreak := False;
  StyleNo := RVData.GetItemStyle(ItemNo);
  Tag     := RVData.GetItemTag(ItemNo);
  // modifying text before the field
  if s1<>'' then
  begin
    RVData.SetItemTextW(ItemNo, s1);
    inc(ItemNo);
    end
  else
  begin
    if RVData.IsFromNewLine(ItemNo) then
    begin
      ParaNo := RVData.GetItemPara(ItemNo);
      BR := RVData.GetItem(ItemNo).BR;
      PageBreak := RVData.GetItem(ItemNo).PageBreakBefore;
    end;
    RVData.DeleteItems(ItemNo, 1);
  end;
  // inserting the field
  RVInsertStringW(RVData, ItemNo, '{'+FieldName+'}',
    GetProtectedStyle(RVData.GetRVStyle, StyleNo), ParaNo, Tag);
  if ParaNo>=0 then
  begin
    RVData.GetItem(ItemNo).BR := BR;
    RVData.GetItem(ItemNo).PageBreakBefore := PageBreak;
  end;
  Pos := RVData.GetOffsAfterItem(ItemNo);
  // modifying text after the field
  if s2<>'' then
    RVInsertStringW(RVData, ItemNo+1, s2, StyleNo, -1, Tag);
end;

end.
